
function [out] = SIMdata_logit(I,J,hh,w,phi,sqr_tau,var_c,alpha_true,AbNNfigure)
%% function [out] = SIMdata_logit(I,J,hh,w,phi,sqr_tau,var_c,alpha_true,AbNNfigure)
%% fr spatial binary logit-Modell with group cluster effects
%% x_{metr} - strukturiert; ohne Intersept
%% Graphische Ausgabe: 1.Bild mit Spatial Effects,
%% 2.Bild mit Cluster Effects, 3.Bild mit Parametric effects,
%% 4.Bild mit Theta_i's, 5.Bild mit Wahrscheinlichkeiten p_i's,
%% 6.Bild mit simuliertem binrem Datensatz,
%% 7.Bild mit empirischen Wahrscheinlichkeiten ber PLZ und
%% 8.Bild mit theoretischen durchschnittlichen Wahrscheinlichkeiten ber PLZ.
%% hh = #Clusters (wird in jeder PLZ mit gleicher Anzahl Wege dargestellt)
%% w = #Wege pro Cluster pro PLZ (konst, gerade)
%% alpha_true=[alpha_kat,alpha_metr]'

%% Bildung der Polygonzgen

K=I*J;     %% K = #PLZ
TOTAL=K*hh*w;
w_pro_plz=hh*w;
polyplzx = zeros(K,4);   %% K Polygonzge fr PLZ-Rechtecke
polyplzy = zeros(K,4);   %% von links nach rechts, von oben nach unten

polyhhx = zeros(K*hh,4); %% K*hh Polygonzge fr Cluster-Rechtecke, je hh
polyhhy = zeros(K*hh,4); %% Stck pro PLZ-Rechteck, von oben nach unten

polywegx = zeros(TOTAL,4); %% K*hh*w Polygonzge fr Weg-Rechtecke, je
polywegy = zeros(TOTAL,4); %% w Stck pro Cluster-Rechteck, von rechts nach links

n_mat = zeros(K,4);    %% Nachbarschaftmatrix n_mat(K,4): es gibt <= 4 Nachbarn fr jede PLZ
n_plz = zeros(K,1);    %% #Nachbarn fr jede PLZ

k=0; h=0; v=0;
for i=1:I
  for j=1:J
    k=k+1;
    polyplzx(k,:)=[j-1,j,j,j-1];
    polyplzy(k,:)=[I-i+1,I-i+1,I-i,I-i];

    %% Erzeugung der Matrix n_mat und des Vektors n_plz. Regulres Gitter, Nachbarschaft der 1. Ordnung
    if j<J & j>1 & i<I &i>1
      n_mat(k,:)=[k-1,k+1,k-J,k+J];
      n_plz(k)=4;
    end;
    if i==1& j==1
      n_mat(k,:)=[k+1,k+J,0,0];
      n_plz(k)=2;
    end;
    if i==1 & j==J
      n_mat(k,:)=[k-1,k+J,0,0];
      n_plz(k)=2;
    end;
    if i==I & j==1
      n_mat(k,:)=[k+1,k-J,0,0];
      n_plz(k)=2;
    end;
    if i==I & j==J
      n_mat(k,:)=[k-1,k-J,0,0];
      n_plz(k)=2;
    end; 
    if i==1& j>1 &j<J
      n_mat(k,:)=[k-1,k+1,k+J,0];
      n_plz(k)=3;
    end;
    if i>1 & i<I & j==J
      n_mat(k,:)=[k-1,k-J,k+J,0];
      n_plz(k)=3;
    end;
    if i==I & j>1 &j<J
      n_mat(k,:)=[k-1,k+1,k-J,0];
      n_plz(k)=3;
    end;
    if i>1 &i<I & j==1
      n_mat(k,:)=[k+1,k-J,k+J,0];
      n_plz(k)=3;
    end; 

    for g=1:hh
      h=h+1;
      polyhhx(h,:)=polyplzx(k,:);
      polyhhy(h,:)=[polyplzy(k,1)-(g-1)/hh,polyplzy(k,2)-(g-1)/hh,polyplzy(k,1)-g/hh,polyplzy(k,2)-g/hh];
      
      for weg=1:w
        v=v+1;
        polywegx(v,:)=[polyhhx(h,1)+(weg-1)/w,polyhhx(h,1)+weg/w,polyhhx(h,4)+weg/w,polyhhx(h,4)+(weg-1)/w];
        polywegy(v,:)=polyhhy(h,:);
      end;
    end;
  end;
end;

sigma=sqrt(var_c);
HHeffekt=normrnd(0,sigma,hh,1);    %% HHeffekt

%% PLZ-Effekt ("neues" Pettitts Model)
Q = zeros(K);    %% Prezisionsmatrix
psi=phi/(1+abs(phi));
for k=1:K
  Q(k,k)=1+abs(psi)*(n_plz(k)-1);
  for j=1:n_plz(k)
    Q(k,n_mat(k,j))=-psi;
  end;
end;

%% Simulation des z.Vektors ~ N(0,tau^2*Q^(-1)) nach Cholinscy-Zerlegung
%% L=chol(Q);
%% z=tau*normrnd(0,1,K,1);
%% PLZeffekt=L\z;

%% Direkte Simulation des z.Vektors ~ N(0,tau^2*Q^(-1))
%% PLZeffekt=mvnrnd(zeros(1,K),(tau^2)*(Q^(-1)),1)';

%% Schnelle Simulation des z.Vektors ~ N(0,tau^2*Q^(-1)) nach Cholesky-Zerlegung
p=symrcm(Q);
BandQ=Q(p,p);
L=chol(BandQ);    %% Minimale Bandbreite der Matrix L liefert die schnellste Berechnung von L\z
z=sqrt(sqr_tau)*normrnd(0,1,K,1);
PLZeffekt(p)=L\z;

%% Erzeugung des Prediktors, des Wahrscheinlichkeitenvektor und des Datensatzes
%% Erzeugung der Designmatrix X
%%%%%%%%%%%%%%%%%%%%%%%%%%%%X=[zeros(TOTAL,4),1+zeros(TOTAL,1)];
X=[zeros(TOTAL,4)];
Theta=zeros(TOTAL,1);Py=zeros(TOTAL,1);Y=zeros(TOTAL,1);
for i=1:TOTAL
  X(i,1)=mod(floor((i-1)/w_pro_plz),K)+1;
  X(i,2)=mod(floor((i-1)/w),hh)+1;
  X(i,3)=mod(floor(2*(i-1)/w),2);
%%%%%%%%%%%%  X(i,4)=normrnd(12,8);
  X(i,4)=1+mod((i-1),23);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%PAReffekt=X(:,[5,4,3])*alpha_true;
PAReffekt=X(:,[3,4])*alpha_true;
%%%HHeffekt=zeros(TOTAL,1);
for i=1:TOTAL
%%%  HHeffekt(i)=normrnd(0,std_error(X(i,2)));
  Theta(i)=PAReffekt(i) + HHeffekt(X(i,2)) + PLZeffekt(X(i,1));
end;

Y_cum=zeros(K,1);
Py_cum=zeros(K,1);
for k=0:(K-1)
  for j=1:w_pro_plz
    Py(k*w_pro_plz+j)=exp(Theta(k*w_pro_plz+j))/(1+exp(Theta(k*w_pro_plz+j)));
    Py_cum(k+1)=Py_cum(k+1)+Py(k*w_pro_plz+j);
    Y(k*w_pro_plz+j)=binornd(1,Py(k*w_pro_plz+j));
    Y_cum(k+1)=Y_cum(k+1)+Y(k*w_pro_plz+j);
  end;
  Py_cum(k+1)=Py_cum(k+1)/w_pro_plz;
  Y_cum(k+1)=Y_cum(k+1)/w_pro_plz;
end;


figure(AbNNfigure-1+1);
colormap(autumn);
for k=1:K
  fill(polyplzx(k,:),polyplzy(k,:),PLZeffekt(k));
  hold on
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title(['Spatial Effects (\phi=',num2str(phi),', \tau^2=',num2str(sqr_tau),')']);
colorbar('vert');

figure(AbNNfigure-1+2)
colormap(autumn);
for g=1:hh
  fill([0,1,1,0],[hh-g+1,hh-g+1,hh-g,hh-g],HHeffekt(g));
  hold on
end;
title(['Cluster Effects (var_c=',num2str(var_c),')']);
colorbar('vert');

figure(AbNNfigure-1+3);
colormap(autumn);
for k=1:TOTAL
  fill(polywegx(k,:),polywegy(k,:),PAReffekt(k));
  hold on
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title(['Parametric Effects (\alpha_{true}=(\alpha_{kat} \alpha_{metr})^t=(',num2str(alpha_true'),')^t)']);
colorbar('vert');

figure(AbNNfigure-1+4);
colormap(autumn);
for k=1:TOTAL
  fill(polywegx(k,:),polywegy(k,:),Theta(k));
  hold on
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title('\Theta_i`s (\Theta_i=par.eff + spatial eff. + cluster eff.)');
colorbar('vert');

figure(AbNNfigure-1+5);
colormap(autumn);
for k=1:TOTAL
  fill(polywegx(k,:),polywegy(k,:),Py(k));
  hold on
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title('p_i`s  ( p_i= exp(\Theta_i)/(1+exp(\Theta_i)) )');
colorbar('vert');

figure(AbNNfigure-1+6);
for k=1:TOTAL
  if Y(k)==1
    fill(polywegx(k,:),polywegy(k,:),[1,1,1]);
    hold on
  else
    fill(polywegx(k,:),polywegy(k,:),[0,0,0]);
    hold on
  end;
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title('Binary Responses');

figure(AbNNfigure-1+7);
colormap(autumn);
for k=1:K
  fill(polyplzx(k,:),polyplzy(k,:),Y_cum(k));
  hold on
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title('Empirical Probabilities over each PLZ');
colorbar('vert');

figure(AbNNfigure-1+8);
colormap(autumn);
for k=1:K
  fill(polyplzx(k,:),polyplzy(k,:),Py_cum(k));
  hold on
end;
for k=1:K
  plot(polyplzx(k,:),polyplzy(k,:),'g');
end;
title('Theoretical Average Probabilities over each PLZ');
colorbar('vert');

out.I=I;
out.J=J;
out.hh=hh;
out.w=w;
out.alpha=alpha_true;
out.phi=phi;
out.sqr_tau=sqr_tau;
out.var_c=var_c;
out.n_plz=n_plz;
out.n_mat=n_mat;
out.b=PLZeffekt';
out.c=HHeffekt;
out.Py=Py;
out.X=X;
out.Y=Y;


%% end function SIMdata_logit.m
