function [out]=sim_logit(data,prior_var_a,sig_a,sig_b,sig_c,sig_psi,R,m,K_MH)
% function [out]=sim_logit(data,prior_var_a,sig_a,sig_b,sig_c,sig_psi,R,m,K_MH)
% MCMC driving function (R iterations) for background operation
% with every m^th iteration recorded (mod(R,m) must be equal 0)
% for spatial binary model with group cluster effects
% was used for sim. studies
% DOESN'T need improvement (now correctly a''_l|a''_1,..,a''_{l-1},a'_{l+1},..,a'_p insteed a''_l|a'_1,..,a'_p )
% phi>0
% n_mat(k) gives neighbors of area k (K areas in a I*J grid)
% n_plz(k)=# of neighbors of area k
% c = cluster parameters
% K_MH=number of iterations for optimizing std. error values for MH proposals
% var_c_true=variance for a-priori density for random cluster effect c(l)
% with non-inform. prior for var_c (= unproper prior ~1/x^2 for inv_cvar). The same for sqr_tau
% sig_c(l)=standard error for proposal density for c(l)
% extract parameter values
% individual updates
% 
% Comparizon Maps - ERROR, if lattice is not regular, i.e. if data.I, data.J do not exist.

tic;
y=data.Y;
X=data.X;
n_plz=data.n_plz;
n_mat=data.n_mat;
alpha_true=data.alpha;
b_true=data.b;
c_true=data.c;
phi_true=data.phi;
sqr_tau_true=data.sqr_tau;
var_c_true=data.var_c;
p=size(alpha_true,1);
K=size(b_true,1);
M=size(c_true,1);
n=size(y,1);
b=zeros(K,1);
alpha=zeros(p,1);
c=zeros(M,1);
prior_inv_var_a=1./prior_var_a;
var_c=1.2*var_c_true;
inv_sqr_tau=1/(1.2*sqr_tau_true);
psi_true=phi_true/(1+abs(phi_true));
psi=0.8*psi_true;
bei_bmean=zeros(K,1);
inv_bvar=zeros(K,1);
inv_cvar=1/var_c;
G=zeros(K);
for i=1:K
  for j=1:n_plz(i)
    G(i,n_mat(i,j))=1;
  end;
end;
D=zeros(K);
for i=1:K
  D(i,i)=n_plz(i)-1;
end;
KERN=D-G;
eigen=eig(-KERN);
for i=1:K
  bei_bmean(i)=psi/(1+abs(psi)*(n_plz(i)-1));
  inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
end;

% set starting values for MCMC
eta_b=b(X(:,1));
eta_c=c(X(:,2));
eta=X(:,3:p+2)*alpha+eta_b+eta_c;
% MCMC loop
R_rec=R/m;
out_b=zeros(R_rec,K);
out_a=zeros(R_rec,p);
out_c=zeros(R_rec,M);
out_var_c=zeros(R_rec,1);
out_sqr_tau=zeros(R_rec,1);
out_phi=zeros(R_rec,1);
out_psi=zeros(R_rec,1);

% find good proposal variances
% try to get acceptance rates between 30-60%
anzahl=0;
for k=1:K_MH
  accepta_cum=zeros(p,1);
  acceptb_cum=zeros(K,1);
  acceptc_cum=zeros(M,1);
  acceptpsi_cum=0;
  for r=1:300
    [b,acceptb]=b_ind(n,y,X,eta,b,n_plz,n_mat,bei_bmean,inv_bvar,sig_b);
    eta=eta-eta_b+b(X(:,1));
    eta_b=b(X(:,1));
    [alpha,accepta,eta]=alpha_car(n,y,X,eta,alpha,prior_inv_var_a,sig_a);
    [c,acceptc]=c_ind(n,y,X,eta,c,inv_cvar,sig_c);
    eta=eta-eta_c+c(X(:,2));
    eta_c=c(X(:,2));
    if c'*c>0
      inv_cvar=gamrnd((M/2)-1,2/(c'*c));
    end;

    bQb=0;
    bKERNb=0;
    if psi>0
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)-b_nah_sum);
      end;
      bQb=psi*bKERNb+(b'*b);
    else
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)+b_nah_sum);
      end;
      bQb=-psi*bKERNb+(b'*b);
    end;
    if bQb>0
      inv_sqr_tau=gamrnd((K/2)-1,2/bQb);
      for i=1:K    
        inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
      end;
    end;
    
    [psi,acceptpsi]=psi_car2(K,psi,inv_sqr_tau,bKERNb,eigen,sig_psi);
    if acceptpsi==1
      for i=1:K
        bei_bmean(i)=psi/(1+abs(psi)*(n_plz(i)-1));
        inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
      end;
    end;

    
    if r>100
      accepta_cum=accepta_cum+accepta;
      acceptb_cum=acceptb_cum+acceptb;
      acceptc_cum=acceptc_cum+acceptc;
      acceptpsi_cum=acceptpsi_cum+acceptpsi;
    end;
  end;
  accepta_cum=accepta_cum/(300-100);
  acceptb_cum=acceptb_cum/(300-100);
  acceptc_cum=acceptc_cum/(300-100);
  acceptpsi_cum=acceptpsi_cum/(300-100);
  for j=1:K
     if acceptb_cum(j)>.6
      sig_b(j)=(1+acceptb_cum(j))*sig_b(j);
     end;
     if acceptb_cum<.3
       sig_b(j)=sig_b(j)*(.5+acceptb_cum(j));
     end;
  end;
  for j=1:p
    if accepta_cum(j)>.6
       sig_a(j)=(1+accepta_cum(j))*sig_a(j);
    end;
    if accepta_cum(j)<.3;
       sig_a(j)=sig_a(j)*(.5+accepta_cum(j));
    end;
  end;
  for j=1:M
    if acceptc_cum(j)>.6
       sig_c(j)=(1+acceptc_cum(j))*sig_c(j);
    end;
    if acceptc_cum(j)<.3;
       sig_c(j)=sig_c(j)*(.5+acceptc_cum(j));
    end;
  end;
  if acceptpsi_cum>.6
       sig_psi=(1+acceptpsi_cum)*sig_psi;
  end;
  if acceptpsi_cum<.3;
       sig_psi=sig_psi*(.5+acceptpsi_cum);
  end;

  anzahl=anzahl+1
end;
% start of MCMC loop
disp('start of MCMC loop    ');
anzahl=0;
accepta_cum=zeros(p,1);
acceptb_cum=zeros(K,1);
acceptc_cum=zeros(M,1);
acceptpsi_cum=0;
for r=1:R
    [b,acceptb]=b_ind(n,y,X,eta,b,n_plz,n_mat,bei_bmean,inv_bvar,sig_b);
    eta=eta-eta_b+b(X(:,1));
    eta_b=b(X(:,1));
    [alpha,accepta,eta]=alpha_car(n,y,X,eta,alpha,prior_inv_var_a,sig_a);
    [c,acceptc]=c_ind(n,y,X,eta,c,inv_cvar,sig_c);
    eta=eta-eta_c+c(X(:,2));
    eta_c=c(X(:,2));
    if c'*c>0
      inv_cvar=gamrnd((M/2)-1,2/(c'*c));
    end;
    bQb=0;
    bKERNb=0;
    if psi>0
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)-b_nah_sum);
      end;
      bQb=psi*bKERNb+(b'*b);
    else
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)+b_nah_sum);
      end;
      bQb=-psi*bKERNb+(b'*b);
    end;
    if bQb>0
      inv_sqr_tau=gamrnd((K/2)-1,2/bQb);
      for i=1:K    
        inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
      end;
    end;
    
    if sig_psi>10
      [psi,acceptpsi]=psi_car2_uni(K,psi,inv_sqr_tau,bKERNb,eigen);
    else
      [psi,acceptpsi]=psi_car2(K,psi,inv_sqr_tau,bKERNb,eigen,sig_psi);
    end;
    if acceptpsi==1
      for i=1:K
        bei_bmean(i)=psi/(1+abs(psi)*(n_plz(i)-1));
        inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
      end;
    end;

    
    if mod(r,m)==0
       anzahl=anzahl+1
       div=r/m;
       out_b(div,:)=b';
       out_a(div,:)=alpha';
       out_c(div,:)=c';
       out_var_c(div)=1/inv_cvar;
       out_sqr_tau(div)=1/inv_sqr_tau;
       out_phi(div)=psi/(1-abs(psi));
       out_psi(div)=psi;

    end;
    accepta_cum=accepta_cum+accepta;
    acceptb_cum=acceptb_cum+acceptb;
    acceptc_cum=acceptc_cum+acceptc;
    acceptpsi_cum=acceptpsi_cum+acceptpsi;
end;
disp('sig_a  ');sig_a'
disp('sig_b  ');sig_b'
disp('sig_c  ');sig_c'
disp('sig_psi  ');sig_psi
disp('accepta_cum   ');accepta_cum/R
disp('acceptb_cum   ');acceptb_cum/R
disp('acceptc_cum   ');acceptc_cum/R
disp('acceptpsi_cum   ');acceptpsi_cum/R
if sig_psi>10
  disp('sig_psi>10 => uni(0,1) for proposal psi is used');
else
  disp('sig_psi<10 => norm(psi_previous,sig_psi) for proposal psi is used');
end;
a_mean=sum(out_a)/R_rec;
b_mean=sum(out_b)/R_rec;
c_mean=sum(out_c)/R_rec;
var_c_mean=sum(out_var_c)/R_rec;
sqr_tau_mean=sum(out_sqr_tau)/R_rec;
psi_mean=sum(out_psi)/R_rec;
phi_mean1=sum(out_phi)/R_rec;
phi_mean2=psi_mean/(1-abs(psi_mean));
a_mode=zeros(p,1);
b_mode=zeros(K,1);
c_mode=zeros(M,1);
var_c_mode=0;
sqr_tau_mode=0;
psi_mode=0;
phi_mode=0;
a_quantile=zeros(p,5);
b_quantile=zeros(K,5);
c_quantile=zeros(M,5);
var_c_quantile=zeros(1,5);
sqr_tau_quantile=zeros(1,5);
psi_quantile=zeros(1,5);
phi_quantile=zeros(1,5);

%figure(1)
for i=1:p
%  subplot(p,1,i)
  figure(i)
  plot(1:R_rec,out_a(:,i),':b');
  hold on 
  plot(1:R_rec,alpha_true(i)+zeros(1,R_rec),'g--','LineWidth',3)
  hold on
  grid on
  title(['\alpha_{',num2str(i),'}']); 
end;

%figure(2)
for i=1:p
%  subplot(p,1,i);
  figure(p+i)
  x=sort(out_a(:,i));
  [h,f,tt]=density(x);
  a_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  a_mode(i)=tt(ind);
  plot(tt,f,':b');
  hold on
  plot(alpha_true(i)+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
  hold on
  plot(a_mode(i)+zeros(1,100),linspace(0,maxim,100),':b');
  hold on
  grid on
  title(['\alpha_{',num2str(i),'}']); 
end;

figure(2*p+1);
for i=1:K
  subplot(floor(sqrt(K))+(K>floor(sqrt(K))*(floor(sqrt(K))+1)),floor(sqrt(K))+(K>(floor(sqrt(K))^2)),i);
  plot(1:R_rec,out_b(:,i),':b');
  hold on 
  plot(1:R_rec,b_true(i)+zeros(1,R_rec),'g--','LineWidth',3)
  hold on
  grid on  
end;

figure(2*p+2);
for i=1:K
  subplot(floor(sqrt(K))+(K>floor(sqrt(K))*(floor(sqrt(K))+1)),floor(sqrt(K))+(K>(floor(sqrt(K))^2)),i);
  x=sort(out_b(:,i));
  [h,f,tt]=density(x);
  b_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  b_mode(i)=tt(ind);
  plot(tt,f,':b');
  hold on
  plot(b_true(i)+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
  hold on
  plot(b_mode(i)+zeros(1,100),linspace(0,maxim,100),':b');
  hold on
  grid on
end;

figure(2*p+3)
%subplot(2,1,1);
plot(1:R_rec,out_sqr_tau,':b');
hold on 
plot(1:R_rec,sqr_tau_true+zeros(1,R_rec),'g--','LineWidth',3)
hold on
grid on
title('simulations of \tau^2');
figure(2*p+4)
%subplot(2,1,2);
x=sort(out_sqr_tau);
[h,f,tt]=density(x);
sqr_tau_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
sqr_tau_mode=tt(ind);
plot(tt,f,':b');
hold on
plot(sqr_tau_true+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
hold on
plot(sqr_tau_mode+zeros(1,100),linspace(0,maxim,100),':b');
hold on
grid on
title('posterior density of \tau^2');

figure(2*p+5)
%subplot(2,1,1);
plot(1:R_rec,out_psi,':b');
hold on 
plot(1:R_rec,psi_true+zeros(1,R_rec),'g--','LineWidth',3)
hold on
grid on
title('simulations of \psi = \phi / (1+|\phi|)');
figure(2*p+6)
%subplot(2,1,2);
x=sort(out_psi);
[h,f,tt]=density(x);
psi_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
psi_mode=tt(ind);
plot(tt,f,':b');
hold on
plot(psi_true+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
hold on
plot(psi_mode+zeros(1,100),linspace(0,maxim,100),':b');
hold on
grid on
title('posterior density of \psi = \phi / (1+|\phi|)');

figure(2*p+7)
%subplot(2,1,1);
plot(1:R_rec,out_phi,':b');
hold on 
plot(1:R_rec,phi_true+zeros(1,R_rec),'g--','LineWidth',3)
hold on
grid on
title('simulations of \phi (transformed from simulations of \psi)'); 

figure(2*p+8)
%subplot(2,1,2);
x=sort(out_phi);
[h,f,tt]=density(x);
phi_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
phi_mode=tt(ind);
plot(tt,f,':b');
hold on
plot(phi_true+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
hold on
plot(phi_mode+zeros(1,100),linspace(0,maxim,100),':b');
hold on
%plot(phi_mode2+zeros(1,100),linspace(0,maxim,100),'r.');
grid on
title('posterior density of \phi');

figure(2*p+9)
for i=1:M
  subplot(floor(sqrt(M))+(M>floor(sqrt(M))*(floor(sqrt(M))+1)),floor(sqrt(M))+(M>(floor(sqrt(M))^2)),i);
  plot(1:R_rec,out_c(:,i),':b');
  hold on 
  plot(1:R_rec,c_true(i)+zeros(1,R_rec),'g--','LineWidth',3)
  hold on
  grid on  
end;

figure(2*p+10)
for i=1:M
  subplot(floor(sqrt(M))+(M>floor(sqrt(M))*(floor(sqrt(M))+1)),floor(sqrt(M))+(M>(floor(sqrt(M))^2)),i);
  x=sort(out_c(:,i));
  [h,f,tt]=density(x);
  c_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  c_mode(i)=tt(ind);
  plot(tt,f,':b');
  hold on
  plot(c_true(i)+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
  hold on
  plot(c_mode(i)+zeros(1,100),linspace(0,maxim,100),':b');
  hold on
  grid on
end;

figure(2*p+11)
%subplot(2,1,1);
plot(1:R_rec,out_var_c,':b');
hold on 
plot(1:R_rec,var_c_true+zeros(1,R_rec),'g--','LineWidth',3)
hold on
grid on
title('simulations of var_c');

figure(2*p+12)
%subplot(2,1,2);
x=sort(out_var_c);
[h,f,tt]=density(x);
var_c_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
var_c_mode=tt(ind);
plot(tt,f,':b');
hold on
plot(var_c_true+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
hold on
plot(var_c_mode+zeros(1,100),linspace(0,maxim,100),':b');
hold on
grid on
title('posterior density of var_c');

I=data.I;
J=data.J;
figure(2*p+13)
subplot(2,2,1)
IJmap(I,J,b_true)
fill([0,J,J,0],[0,0,-1,-1],max([max(b_true),max(b_mode),max(b_mean),max(b_quantile(:,3))]));
fill([0,J,J,0],[I+1,I+1,I,I],min([min(b_true),min(b_mode),min(b_mean),min(b_quantile(:,3))]));
colorbar('vert');
axis([0 J 0 I])
title('true spatial effects b`s');
hold off
subplot(2,2,2)
IJmap(I,J,b_mode)
fill([0,J,J,0],[0,0,-1,-1],max([max(b_true),max(b_mode),max(b_mean),max(b_quantile(:,3))]));
fill([0,J,J,0],[I+1,I+1,I,I],min([min(b_true),min(b_mode),min(b_mean),min(b_quantile(:,3))]));
colorbar('vert');
axis([0 J 0 I])
title('their estimators (posterior modes)');
hold off
subplot(2,2,3)
IJmap(I,J,b_mean)
fill([0,J,J,0],[0,0,-1,-1],max([max(b_true),max(b_mode),max(b_mean),max(b_quantile(:,3))]));
fill([0,J,J,0],[I+1,I+1,I,I],min([min(b_true),min(b_mode),min(b_mean),min(b_quantile(:,3))]));
colorbar('vert');
axis([0 J 0 I])
title('their estimators (posterior means)');
hold off
subplot(2,2,4)
IJmap(I,J,b_quantile(:,3))
fill([0,J,J,0],[0,0,-1,-1],max([max(b_true),max(b_mode),max(b_mean),max(b_quantile(:,3))]));
fill([0,J,J,0],[I+1,I+1,I,I],min([min(b_true),min(b_mode),min(b_mean),min(b_quantile(:,3))]));
colorbar('vert');
axis([0 J 0 I])
title('their estimators (posterior medians)');
hold off

figure(2*p+14)
subplot(2,2,1)
IJmap(M,1,c_true)
fill([0,1,1,0],[0,0,-1,-1],max([max(c_true),max(c_mode),max(c_mean),max(c_quantile(:,3))]));
fill([0,1,1,0],[M+1,M+1,M,M],min([min(c_true),min(c_mode),min(c_mean),min(c_quantile(:,3))]));
colorbar('vert');
title('true cluster effects c`s');
axis([0 1 0 M])
hold off
subplot(2,2,2)
IJmap(M,1,c_mode)
fill([0,1,1,0],[0,0,-1,-1],max([max(c_true),max(c_mode),max(c_mean),max(c_quantile(:,3))]));
fill([0,1,1,0],[M+1,M+1,M,M],min([min(c_true),min(c_mode),min(c_mean),min(c_quantile(:,3))]));
colorbar('vert');
title('their estimators (posterior modes)');
axis([0 1 0 M])
hold off
subplot(2,2,3)
IJmap(M,1,c_mean)
fill([0,1,1,0],[0,0,-1,-1],max([max(c_true),max(c_mode),max(c_mean),max(c_quantile(:,3))]));
fill([0,1,1,0],[M+1,M+1,M,M],min([min(c_true),min(c_mode),min(c_mean),min(c_quantile(:,3))]));
colorbar('vert');
title('their estimators (posterior means)');
axis([0 1 0 M])
hold off
subplot(2,2,4)
fill([0,1,1,0],[0,0,-1,-1],max([max(c_true),max(c_mode),max(c_mean),max(c_quantile(:,3))]));
fill([0,1,1,0],[M+1,M+1,M,M],min([min(c_true),min(c_mode),min(c_mean),min(c_quantile(:,3))]));
colorbar('vert');
IJmap(M,1,c_quantile(:,3))
title('their estimators (posterior medians)');
axis([0 1 0 M])
hold off

figure(2*p+15)
for i=1:p
  subplot(1,p,i)
  acf_plot(out_a(:,i),floor(R_rec/10),['\alpha_{',num2str(i),'}']);
  hold off;
end;

figure(2*p+16)
subplot(1,3,1)
acf_plot(out_sqr_tau,floor(R_rec/10),'\tau^2');
hold off
subplot(1,3,2)
acf_plot(out_psi,floor(R_rec/10),'\psi');
hold off
subplot(1,3,3)
acf_plot(out_var_c,floor(R_rec/10),'var_c');
hold off

out.data=data;
out.out_a=out_a;
out.out_b=out_b;
out.out_c=out_c;
out.out_var_c=out_var_c;
out.out_sqr_tau=out_sqr_tau;
out.out_psi=out_psi;
% out.out_phi=out.out_psi./(1+abs(out.out_psi));
out.a_mean=a_mean';
out.b_mean=b_mean';
out.c_mean=c_mean';
out.var_c_mean=var_c_mean;
out.sqr_tau_mean=sqr_tau_mean;
out.psi_mean=psi_mean;
out.phi_mean1=phi_mean1;
out.phi_mean2=phi_mean2;
out.a_mode=a_mode;
out.b_mode=b_mode;
out.c_mode=c_mode;
out.var_c_mode=var_c_mode;
out.sqr_tau_mode=sqr_tau_mode;
out.psi_mode=psi_mode;
out.phi_mode=phi_mode;
out.a_quantile=a_quantile;
out.b_quantile=b_quantile;
out.c_quantile=c_quantile;
out.var_c_quantile=var_c_quantile;
out.sqr_tau_quantile=sqr_tau_quantile;
out.psi_quantile=psi_quantile;
out.phi_quantile=phi_quantile;
out.sig_a=sig_a;
out.sig_b=sig_b;
out.sig_c=sig_c;
out.sig_psi=sig_psi;
out.accept_a=accepta_cum/R;
out.accept_b=acceptb_cum/R;
out.accept_c=acceptc_cum/R;
out.accept_psi=acceptpsi_cum/R;
out.R=R;
out.m=m;
out.K_MH=K_MH;

toc;
t=toc
out.t=t;

% end of function sim_logit.m
