function [out]=sim_logit_indiv(data,prior_var_a,prior_var_sigma,sig_a,sig_b,sig_sigma,sig_psi,R,step,K_MH)

% function [out]=sim_logit_indiv(data,prior_var_a,prior_var_sigma,sig_a,sig_b,sig_sigma,sig_psi,R,step,K_MH)
% MCMC driving function (R iterations) for background operation
% with every step^th iteration recorded (mod(R,step) must be equal 0)
% for spatial binary logit-Modell with individual cluster effects
% was used for application (new compared with sim.study: \psi <>0, output plots; etc.?)
% No latent variables Z, therefore MH-step is needed for all parameters except \tau^2
% K_MH=number of iterations for optimizing std. error value for MH proposals
% and for burn in phase
% sigma_true=now vector of the true M cluster parameters, sigma_true(1)=1
% all {%%%%%%%%%% sites and} clusters must be presented
% extract parameter values
% individual updates
% 24.09.02: neu Term 1.2 in sigma(2:M)=1.2*sigma_true(2:M);
% 28.10.02: neu: term3 in phi-updates (macht [\phi] \propto 1/(1+\phi)^1.5 - proper)
%%%%%%%%%% 05.06.03: neu: term3 macht [\phi] \propto 1/(1+x)^2.5 - proper

tic;

y=data.Y;
X=data.X;
n_plz=data.n_plz;
n_mat=data.n_mat;

G=data.G;

alpha_true=data.alpha;
alpha_lab=data.alpha_lab;
b_true=data.b;
sigma_true=data.sigma;
phi_true=data.phi;
sqr_tau_true=data.sqr_tau;
inv_prior_var_a=1./prior_var_a;
inv_prior_var_sigma=1/prior_var_sigma;

p=size(alpha_true,1);
K=size(b_true,1);
M=data.M;
n=size(y,1);


%%%%%%%%%% G=zeros(K);
%%%%%%%%%% for i=1:K
%%%%%%%%%%   for j=1:n_plz(i)
%%%%%%%%%%     G(i,n_mat(i,j))=1;
%%%%%%%%%%   end;
%%%%%%%%%% end;

D=zeros(K);
for i=1:K
  D(i,i)=n_plz(i)-1;
end;
%%%%%%%%%% KERN=D-G; eigen=eig(-KERN);
eigen_pos=eig(G-D);
eigen_neg=eig(G+D);

% set starting values for MCMC

b=zeros(K,1);
alpha=zeros(p,1);

sigma=zeros(M,1);
sigma(1)=sigma_true(1);
sigma(2:M)=1.2*sigma_true(2:M);
inv_sqr_tau=1/(1.2*sqr_tau_true);
psi_true=phi_true/(1+abs(phi_true));
psi=0.8*psi_true;
bei_bmean=zeros(K,1);
inv_bvar=zeros(K,1);
for i=1:K
  bei_bmean(i)=psi/(1+abs(psi)*(n_plz(i)-1));
  inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
end;


eta_a=X(:,[3:p+2])*alpha;
eta_b=b(X(:,1));
eta=(eta_a+eta_b)./sigma(X(:,2));

% MCMC loop

R_rec=R/step;
out_b=zeros(R_rec,K);
out_a=zeros(R_rec,p);
out_sigma=zeros(R_rec,M);
out_sqr_tau=zeros(R_rec,1);
out_phi=zeros(R_rec,1);
out_psi=zeros(R_rec,1);

% find good proposal variances
% try to get acceptance rates between 30-60%
anzahl=0;
for k=1:K_MH
  accepta_cum=zeros(p,1);
  acceptb_cum=zeros(K,1);
  acceptsigma_cum=zeros(M,1);
  acceptpsi_cum=0;
  for r=1:300
    [b,acceptb]=b_ind_logit_neu(n,y,X,eta,b,n_plz,n_mat,bei_bmean,inv_bvar,sig_b,sigma);
    eta=eta+(b(X(:,1))-eta_b)./sigma(X(:,2));
    eta_b=b(X(:,1));
    [alpha,accepta,eta]=alpha_logit_neu(n,y,X,eta,alpha,inv_prior_var_a,sig_a,sigma);
    [sigma,acceptsigma,eta]=sigma_logit_neu(n,M,y,X,eta,sigma,inv_prior_var_sigma,sig_sigma);
    
    bQb=0;
    bKERNb=0;
    if psi>0
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)-b_nah_sum);
      end;
      bQb=psi*bKERNb+(b'*b);
    else
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)+b_nah_sum);
      end;
      bQb=-psi*bKERNb+(b'*b);
    end;
    if bQb>0
      inv_sqr_tau=gamrnd((K/2)-1,2/bQb);
      for i=1:K    
        inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
      end;
    end;
    
    [psi,acceptpsi]=psi_car3_both(K,psi,inv_sqr_tau,bKERNb,eigen_pos,eigen_neg,sig_psi,n_plz,n_mat,b);
    if acceptpsi==1
      for i=1:K
        bei_bmean(i)=psi/(1+abs(psi)*(n_plz(i)-1));
        inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
      end;
    end;

    
    if r>100
      accepta_cum=accepta_cum+accepta;
      acceptb_cum=acceptb_cum+acceptb;
      acceptsigma_cum=acceptsigma_cum+acceptsigma;
      acceptpsi_cum=acceptpsi_cum+acceptpsi;
    end;
  end;
  accepta_cum=accepta_cum/(300-100);
  acceptb_cum=acceptb_cum/(300-100);
  acceptsigma_cum=acceptsigma_cum/(300-100);
  acceptpsi_cum=acceptpsi_cum/(300-100);
  for j=1:K
     if acceptb_cum(j)>.6
      sig_b(j)=(1+acceptb_cum(j))*sig_b(j);
     end;
     if acceptb_cum<.3
       sig_b(j)=sig_b(j)*(.5+acceptb_cum(j));
     end;
  end;
  for j=1:p
    if accepta_cum(j)>.6
       sig_a(j)=(1+accepta_cum(j))*sig_a(j);
    end;
    if accepta_cum(j)<.3;
       sig_a(j)=sig_a(j)*(.5+accepta_cum(j));
    end;
  end;
  for j=2:M
    if acceptsigma_cum(j)>.6
       sig_sigma(j)=(1+acceptsigma_cum(j))*sig_sigma(j);
    end;
    if acceptsigma_cum(j)<.3;
       sig_sigma(j)=sig_sigma(j)*(.5+acceptsigma_cum(j));
    end;
  end;
  if acceptpsi_cum>.6
       sig_psi=(1+acceptpsi_cum)*sig_psi;
  end;
  if acceptpsi_cum<.3;
       sig_psi=sig_psi*(.5+acceptpsi_cum);
  end;

  anzahl=anzahl+1
end;

% start of MCMC loop

disp('start of MCMC loop    ');
anzahl=0;
accepta_cum=zeros(p,1);
acceptb_cum=zeros(K,1);
acceptsigma_cum=zeros(M,1);
acceptpsi_cum=0;
for r=1:R
  [b,acceptb]=b_ind_logit_neu(n,y,X,eta,b,n_plz,n_mat,bei_bmean,inv_bvar,sig_b,sigma);
  eta=eta+(b(X(:,1))-eta_b)./sigma(X(:,2));
  eta_b=b(X(:,1));
  [alpha,accepta,eta]=alpha_logit_neu(n,y,X,eta,alpha,inv_prior_var_a,sig_a,sigma);
  [sigma,acceptsigma,eta]=sigma_logit_neu(n,M,y,X,eta,sigma,inv_prior_var_sigma,sig_sigma);
  
  bQb=0;
  bKERNb=0;
  if psi>0
    for l=1:K
      b_nah_sum=0;
      for k=1:n_plz(l)
        b_nah_sum=b_nah_sum+b(n_mat(l,k));
      end;
      bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)-b_nah_sum);
    end;
    bQb=psi*bKERNb+(b'*b);
  else
    for l=1:K
      b_nah_sum=0;
      for k=1:n_plz(l)
        b_nah_sum=b_nah_sum+b(n_mat(l,k));
      end;
      bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)+b_nah_sum);
    end;
    bQb=-psi*bKERNb+(b'*b);
  end;
  if bQb>0
    inv_sqr_tau=gamrnd((K/2)-1,2/bQb);
    for i=1:K    
      inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
    end;
  end;
  
  if sig_psi>10
    [psi,acceptpsi]=psi_car3_uni_both(K,psi,inv_sqr_tau,bKERNb,eigen_pos,eigen_neg,n_plz,n_mat,b);
  else
    [psi,acceptpsi]=psi_car3_both(K,psi,inv_sqr_tau,bKERNb,eigen_pos,eigen_neg,sig_psi,n_plz,n_mat,b);
  end;
  if acceptpsi==1
    for i=1:K
      bei_bmean(i)=psi/(1+abs(psi)*(n_plz(i)-1));
      inv_bvar(i)=(1+abs(psi)*(n_plz(i)-1))*inv_sqr_tau;
    end;
  end;

  if mod(r,step)==0
    anzahl=anzahl+1
    div=r/step;
    out_b(div,:)=b';
    out_a(div,:)=alpha';
    out_sigma(div,:)=sigma';
    out_sqr_tau(div)=1/inv_sqr_tau;
    out_phi(div)=psi/(1-abs(psi));
    out_psi(div)=psi;
  end;
  accepta_cum=accepta_cum+accepta;
  acceptb_cum=acceptb_cum+acceptb;
  acceptsigma_cum=acceptsigma_cum+acceptsigma;
  acceptpsi_cum=acceptpsi_cum+acceptpsi;
end;
disp('sig_a  ');sig_a'
disp('sig_b  ');sig_b'
disp('sig_sigma  ');sig_sigma'
disp('sig_psi  ');sig_psi
accepta_cum=accepta_cum/R;
acceptb_cum=acceptb_cum/R;
acceptsigma_cum=acceptsigma_cum/R;
acceptpsi_cum=acceptpsi_cum/R;
disp('accepta_cum   ');accepta_cum
disp('acceptb_cum   ');acceptb_cum
disp('acceptsigma_cum   ');acceptsigma_cum
disp('acceptpsi_cum   ');acceptpsi_cum

if sig_psi>10
  disp('sig_psi>10 => uni(0,1) for proposal psi is used');
else
  disp('sig_psi<10 => norm(psi_previous,sig_psi) for proposal psi is used');
end;

a_mean=sum(out_a)/R_rec;
b_mean=sum(out_b)/R_rec;
sigma_mean=sum(out_sigma)/R_rec;
sqr_tau_mean=sum(out_sqr_tau)/R_rec;
psi_mean=sum(out_psi)/R_rec;
phi_mean1=sum(out_phi)/R_rec;
phi_mean2=psi_mean/(1-abs(psi_mean));
a_mode=zeros(p,1);
b_mode=zeros(K,1);
sigma_mode=zeros(M,1);
sqr_tau_mode=0;
psi_mode=0;
phi_mode=0;
a_quantile=zeros(p,5);
b_quantile=zeros(K,5);
sigma_quantile=zeros(M,5);
sqr_tau_quantile=zeros(1,5);
psi_quantile=zeros(1,5);
phi_quantile=zeros(1,5);

figure(1)
for i=1:p
  subplot(floor(sqrt(p))+(p>floor(sqrt(p))*(floor(sqrt(p))+1)),floor(sqrt(p))+(p>(floor(sqrt(p))^2)),i);
%%%%%%%%%%   figure(i)
  plot(1:R_rec,out_a(:,i),'b');
  hold on 
  plot(1:R_rec,alpha_true(i)+zeros(1,R_rec),'g--')
  hold on
%%%%%%%%%%   grid on
  title(['\fontsize{6}',alpha_lab(i,:)]);
  if i~=p
    set(gca,'XTickLabel',[]);
  end
end;

figure(2)
for i=1:p
  subplot(floor(sqrt(p))+(p>floor(sqrt(p))*(floor(sqrt(p))+1)),floor(sqrt(p))+(p>(floor(sqrt(p))^2)),i);
%%%%%%%%%%   figure(p+i)
  x=sort(out_a(:,i));
  [h,f,tt]=density(x);
  a_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  a_mode(i)=tt(ind);
  plot(tt,f,'b');
  hold on
  axis([tt(1) tt(max(size(tt))) 0 1.05*maxim])
%  plot(alpha_true(i)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(a_mode(i)+zeros(1,100),linspace(0,maxim,100),'b');
  plot(a_quantile(i,2)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(a_quantile(i,4)+zeros(1,100),linspace(0,maxim,100),'g--');
%%%%%%%%%%   grid on
  set(gca,'FontSize',6);
  title(['\fontsize{6}',alpha_lab(i,:)]); 
end;
%%%%%%%%%% adjust = .85;
%%%%%%%%%% h = findobj(gcf,'type','axes');
%%%%%%%%%% for i = 1:length(h)
%%%%%%%%%%   pos = get(h(i),'position');
%%%%%%%%%%   pos(4) = pos(4)*adjust;
%%%%%%%%%%   set(h(i),'position',pos);
%%%%%%%%%% end;

figure(3);
for i=1:K
  subplot(floor(sqrt(K))+(K>floor(sqrt(K))*(floor(sqrt(K))+1)),floor(sqrt(K))+(K>(floor(sqrt(K))^2)),i);
  plot(1:R_rec,out_b(:,i),'b');
  hold on 
  plot(1:R_rec,b_true(i)+zeros(1,R_rec),'g--')
  hold on
%%%%%%%%%%   grid on
  if i~=K
    set(gca,'XTickLabel',[]);
  end
end;

figure(4);
for i=1:K
  subplot(floor(sqrt(K))+(K>floor(sqrt(K))*(floor(sqrt(K))+1)),floor(sqrt(K))+(K>(floor(sqrt(K))^2)),i);
  x=sort(out_b(:,i));
  [h,f,tt]=density(x);
  b_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  b_mode(i)=tt(ind);
  plot(tt,f,'b');
  axis([tt(1) tt(max(size(tt))) 0 1.05*maxim])
  hold on
%%%%%%%%%%   plot(b_true(i)+zeros(1,100),linspace(0,maxim,100),'g--');
%%%%%%%%%%   hold on
  plot(b_mode(i)+zeros(1,100),linspace(0,maxim,100),'b');
  plot(b_quantile(i,2)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(b_quantile(i,4)+zeros(1,100),linspace(0,maxim,100),'g--');
%%%%%%%%%%   hold on
%%%%%%%%%%   grid on
  set(gca,'FontSize',6);
end;
%%%%%%%%%% h = findobj(gcf,'type','axes');
%%%%%%%%%% for i = 1:length(h)
%%%%%%%%%%   pos = get(h(i),'position');
%%%%%%%%%%   pos(4) = pos(4)*adjust;
%%%%%%%%%%   set(h(i),'position',pos);
%%%%%%%%%% end;

figure(5)
subplot(2,1,1);
plot(1:R_rec,out_sqr_tau,'b');
hold on 
%%%%%%%%%% plot(1:R_rec,sqr_tau_true+zeros(1,R_rec),'g--')
grid on
title('simulations of \tau^2');
subplot(2,1,2);
plot(1:R_rec,out_psi,'b');
hold on 
plot(1:R_rec,psi_true+zeros(1,R_rec),'g--')
grid on
title('simulations of \psi = \phi / (1+|\phi|)');

figure(6)
subplot(1,2,1);
x=sort(out_sqr_tau);
[h,f,tt]=density(x);
sqr_tau_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
sqr_tau_mode=tt(ind);
plot(tt,f,'b');
hold on
%%%%%%%%%% plot(sqr_tau_true+zeros(1,100),linspace(0,maxim,100),'g--');
plot(sqr_tau_mode+zeros(1,100),linspace(0,maxim,100),'b');
plot(sqr_tau_quantile(2)+zeros(1,100),linspace(0,maxim,100),'g--');
plot(sqr_tau_quantile(4)+zeros(1,100),linspace(0,maxim,100),'g--');
grid on
title('posterior density of \tau^2');
subplot(1,2,2);
x=sort(out_psi);
[h,f,tt]=density(x);
psi_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
psi_mode=tt(ind);
plot(tt,f,'b');
hold on
axis([-1.2 1.2 0 1.05*maxim])
%%%%%%%%%% %%%%%%%%%% plot(psi_true+zeros(1,100),linspace(0,maxim,100),'g--');
plot(psi_mode+zeros(1,100),linspace(0,maxim,100),'b');
plot(psi_quantile(2)+zeros(1,100),linspace(0,maxim,100),'g--');
plot(psi_quantile(4)+zeros(1,100),linspace(0,maxim,100),'g--');
grid on
title('posterior density of \psi = \phi / (1+|\phi|)');

figure(7)
subplot(2,1,1);
plot(1:R_rec,out_phi,'b');
hold on 
plot(1:R_rec,phi_true+zeros(1,R_rec),'g--')
hold on
grid on
title('simulations of \phi (transformed from simulations of \psi)'); 
subplot(2,1,2);
x=sort(out_phi);
[h,f,tt]=density(x);
phi_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
phi_mode=tt(ind);
plot(tt,f,'b');
hold on
plot(phi_true+zeros(1,100),linspace(0,maxim,100),'g--');
plot(phi_mode+zeros(1,100),linspace(0,maxim,100),'b');
%plot(phi_mode2+zeros(1,100),linspace(0,maxim,100),'r.');
grid on
title('posterior density of \phi');

figure(8)
for i=2:M
  subplot(floor(sqrt(M-1))+(M-1>floor(sqrt(M-1))*(floor(sqrt(M-1))+1)),floor(sqrt(M-1))+(M-1>(floor(sqrt(M-1))^2)),i-1);
  plot(1:R_rec,out_sigma(:,i),'b');
  hold on 
%%%%%%%%%%   plot(1:R_rec,sigma_true(i)+zeros(1,R_rec),'g--','LineWidth',3)
%%%%%%%%%%   grid on
%%%%%%%%%%   hold off
  title(['\sigma_{',num2str(i),'}']);
  if i~=M
    set(gca,'XTickLabel',[]);
  end
end;

figure(9)
for i=2:M
  subplot(floor(sqrt(M-1))+(M-1>floor(sqrt(M-1))*(floor(sqrt(M-1))+1)),floor(sqrt(M-1))+(M-1>(floor(sqrt(M-1))^2)),i-1);
  x=sort(out_sigma(:,i));
  [h,f,tt]=density(x);
  sigma_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  sigma_mode(i)=tt(ind);
  plot(tt,f,'b');
  hold on
  axis([tt(1) tt(max(size(tt))) 0 1.05*maxim])
%%%%%%%%%%   plot(sigma_true(i)+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
%%%%%%%%%%   hold on
  plot(sigma_mode(i)+zeros(1,100),linspace(0,maxim,100),'b');
  plot(sigma_quantile(i,2)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(sigma_quantile(i,4)+zeros(1,100),linspace(0,maxim,100),'g--');
%%%%%%%%%%   hold on
%%%%%%%%%%   grid on
  title(['\sigma_{',num2str(i),'}']);
end;

%%%%%%%%%% 
%%%%%%%%%% HIER BLOCK MIT COMPARIZON MAPS FR b UND c FR REGULRE GITTER
%%%%%%%%%% 

figure(10)
for i=1:p
  subplot(floor(sqrt(p))+(p>floor(sqrt(p))*(floor(sqrt(p))+1)),floor(sqrt(p))+(p>(floor(sqrt(p))^2)),i);
  acf_plot(out_a(:,i),floor(R_rec/10),['\fontsize{6}',alpha_lab(i,:)]);
  set(gca,'FontSize',6);
  grid off;
  if i~=p
    set(gca,'XTickLabel',[]);
  end;
end;
%%%%%%%%%% h = findobj(gcf,'type','axes');
%%%%%%%%%% for i = 1:length(h)
%%%%%%%%%%   pos = get(h(i),'position');
%%%%%%%%%%   pos(4) = pos(4)*adjust;
%%%%%%%%%%   set(h(i),'position',pos);
%%%%%%%%%% end;

figure(11)
subplot(1,2,1)
acf_plot(out_sqr_tau,floor(R_rec/10),'\tau^2');
%%%%%%%%%% hold off
subplot(1,2,2)
acf_plot(out_psi,floor(R_rec/10),'\psi');


figure(12)
for i=2:M
  subplot(1,M-1,i-1)
  acf_plot(out_sigma(:,i),floor(R_rec/10),['\sigma_{',num2str(i),'}']);
  hold off;
end;


out.data=data;
out.out_a=out_a;
out.out_b=out_b;
out.out_sigma=out_sigma;
out.out_sqr_tau=out_sqr_tau;
out.out_psi=out_psi;
% out.out_phi=out.out_psi./(1+abs(out.out_psi));
out.a_mean=a_mean';
out.b_mean=b_mean';
out.sigma_mean=sigma_mean';
out.sqr_tau_mean=sqr_tau_mean;
out.psi_mean=psi_mean;
out.phi_mean1=phi_mean1;
out.phi_mean2=phi_mean2;
out.a_mode=a_mode;
out.b_mode=b_mode;
out.sigma_mode=sigma_mode;
out.sqr_tau_mode=sqr_tau_mode;
out.psi_mode=psi_mode;
out.phi_mode=phi_mode;
out.a_quantile=a_quantile;
out.b_quantile=b_quantile;
out.sigma_quantile=sigma_quantile;
out.sqr_tau_quantile=sqr_tau_quantile;
out.psi_quantile=psi_quantile;
out.phi_quantile=phi_quantile;
out.sig_a=sig_a;
out.sig_b=sig_b;
out.sig_sigma=sig_sigma;
out.sig_psi=sig_psi;
out.accepta_cum=accepta_cum;
out.acceptb_cum=acceptb_cum;
out.acceptsigma_cum=acceptsigma_cum;
out.acceptpsi_cum=acceptpsi_cum;
out.R=R;
out.step=step;
out.K_MH=K_MH;
toc;
t=toc
out.t=t;


% end of function sim_logit_indiv
