function [out]=sim_probit_neu_latent(data,prior_var_a,sig_psi,R,step,K_MH)
% function [out]=sim_probit_neu_latent(data,prior_var_a,sig_psi,R,step,K_MH)
% MCMC driving function (R iterations) for background operation
% with every step^th iteration recorded (mod(R,step) must be equal 0)
% for spatial binary probit-Modell with individual cluster effects
% was used for sim. studies
% MH-step is needed for \psi only
% K_MH=number of iterations for optimizing std. error value for MH proposals
% and for burn in phase
% sqr_sigma_true=now vector of the true M cluster parameters, sqr_sigma_true(1)=1
% all sites and clusters must be presented
% extract parameter values
% joint updates
%%%%%%%%%% 20.5.03: psi_car(_uni)_both statt psi_car(_uni)
%%%%%%%%%% 05.06.03: neu: term3 macht [\phi] \propto 1/(1+x)^2.5 - proper

tic;

y=data.Y;
X=data.X;
n_plz=data.n_plz;
n_mat=data.n_mat;

G=data.G;

alpha_true=data.alpha;
alpha_lab=data.alpha_lab;
b_true=data.b;
sqr_sigma_true=data.sigma.^2;
phi_true=data.phi;
sqr_tau_true=data.sqr_tau;
inv_prior_var_a=1./prior_var_a;

p=size(alpha_true,1);
K=size(b_true,1);
M=data.M;
n=size(y,1);


% H=size(c_true,1);
C=0;
trips_per_cluster=zeros(M,1);
for i=1:n
  trips_per_cluster(X(i,2))=trips_per_cluster(X(i,2))+1;
  C(X(i,2),trips_per_cluster(X(i,2)))=i;
end;
if size(C,1)<M
  disp('nicht alle Clusters sind mit den Wegen dargestellt');
end;

%%%%%%%%%% G=zeros(K);
%%%%%%%%%% for i=1:K
%%%%%%%%%%   for j=1:n_plz(i)
%%%%%%%%%%     G(i,n_mat(i,j))=1;
%%%%%%%%%%   end;
%%%%%%%%%% end;

D=zeros(K);
for i=1:K
  D(i,i)=n_plz(i)-1;
end;
%%%%%%%%%% eigen=eig(-KERN);
KERN_pos=G-D;
KERN_neg=D+G;
eigen_pos=eig(KERN_pos);
eigen_neg=eig(KERN_neg);

% set starting values for MCMC

Z=zeros(n,1);
b=zeros(K,1);
alpha=zeros(p,1);

sqr_sigma(1)=sqr_sigma_true(1);
sqr_sigma(2:M)=1.2*sqr_sigma_true(2:M);
inv_sqr_tau=1/(1.2*sqr_tau_true);
psi_true=phi_true/(1+abs(phi_true));
psi=0.8*psi_true;

if psi>0
  Q=eye(K)-psi*KERN_pos;
else
  Q=eye(K)-psi*KERN_neg;
end;

%new_order=symrcm(Q);

sigma=sqrt(sqr_sigma(X(:,2)));
invSigma=diag(1./sqr_sigma(X(:,2)));
BinvSigma=zeros(n,K);
for i=1:n
  BinvSigma(i,X(i,1))=invSigma(i,i);
end;
BinvSigma=BinvSigma';
BinvSigmaB=zeros(K);
for i=1:n
  BinvSigmaB(X(i,1),X(i,1))=BinvSigmaB(X(i,1),X(i,1))+invSigma(i,i);
end;

eta_a=X(:,[3:p+2])*alpha;
eta=eta_a+b(X(:,1));
eta_Z_b=Z+b(X(:,1));
eta_Z_a=Z+eta_a;

% MCMC loop

R_rec=R/step;
out_Z=zeros(R_rec,n);
out_b=zeros(R_rec,K);
out_a=zeros(R_rec,p);
out_sqr_sigma=zeros(R_rec,M);
out_sqr_tau=zeros(R_rec,1);
out_phi=zeros(R_rec,1);
out_psi=zeros(R_rec,1);

% find good proposal variances
% try to get acceptance rates between 30-60%
anzahl=0;
for k=1:K_MH
  acceptpsi_cum=0;
  for r=1:300
    Z = zbinidp_update_probit(y,n,eta,sigma);
    eta_Z_b=Z+b(X(:,1));
    alpha = regbinidpinf_update_probit(X, eta_Z_b, invSigma,inv_prior_var_a,p)';
    eta_a=X(:,[3:p+2])*alpha;
    eta_Z_a=Z+eta_a;
    b=b_joint_probit(BinvSigmaB,eta_Z_a,BinvSigma,Q,inv_sqr_tau,K);
    eta=eta_a+b(X(:,1));
    eta_Z=eta+Z;
%%%%%%%%%%     inv_sqr_tau=gamrnd((K/2)-1,2/(b'*Q*b));
%%%%%%%%%%     bKERNb=b'*KERN*b;
%%%%%%%%%%     [psi,acceptpsi]=psi_car2(K,psi,inv_sqr_tau,bKERNb,eigen,sig_psi);

    bQb=0;
    bKERNb=0;
    if psi>0
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)-b_nah_sum);
      end;
      bQb=psi*bKERNb+(b'*b);
    else
      for l=1:K
        b_nah_sum=0;
        for k=1:n_plz(l)
          b_nah_sum=b_nah_sum+b(n_mat(l,k));
        end;
        bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)+b_nah_sum);
      end;
      bQb=-psi*bKERNb+(b'*b);
    end;
    if bQb>0
      inv_sqr_tau=gamrnd((K/2)-1,2/bQb);
    end;
    
    [psi,acceptpsi]=psi_car3_both(K,psi,inv_sqr_tau,bKERNb,eigen_pos,eigen_neg,sig_psi,n_plz,n_mat,b);

    if acceptpsi==1
%%%%%%%%%%       Q=eye(K)+psi*KERN;
      if psi>0
        Q=eye(K)-psi*KERN_pos;
      else
        Q=eye(K)-psi*KERN_neg;
      end;
    end;
    if r>100
      acceptpsi_cum=acceptpsi_cum+acceptpsi;
    end;
    sqr_sigma=sqr_sigma_probit(sqr_sigma,eta_Z,C,trips_per_cluster,M);
    sigma=sqrt(sqr_sigma(X(:,2)));
    invSigma=diag(1./sqr_sigma(X(:,2)));
    BinvSigma=zeros(n,K);
    for i=1:n
      BinvSigma(i,X(i,1))=invSigma(i,i);
    end;
    BinvSigma= BinvSigma';
    BinvSigmaB=zeros(K);
    for i=1:n
      BinvSigmaB(X(i,1),X(i,1))=BinvSigmaB(X(i,1),X(i,1))+invSigma(i,i);
    end;

  end;
  acceptpsi_cum=acceptpsi_cum/(300-100);
  if acceptpsi_cum>.6
       sig_psi=(1+acceptpsi_cum)*sig_psi;
  end;
  if acceptpsi_cum<.3;
       sig_psi=sig_psi*(.5+acceptpsi_cum);
  end;

  anzahl=anzahl+1
end;

% start of MCMC loop
disp('start of MCMC loop    ');
anzahl=0;
acceptpsi_cum=0;
for r=1:R
  Z = zbinidp_update_probit(y,n,eta,sigma);
  eta_Z_b=Z+b(X(:,1));
  alpha = regbinidpinf_update_probit(X, eta_Z_b, invSigma,inv_prior_var_a,p)';
  eta_a=X(:,[3:p+2])*alpha;
  eta_Z_a=Z+eta_a;
  b=b_joint_probit(BinvSigmaB,eta_Z_a,BinvSigma,Q,inv_sqr_tau,K);
  eta=eta_a+b(X(:,1));
  eta_Z=eta+Z;
%%%%%%%%%%   inv_sqr_tau=gamrnd((K/2)-1,2/(b'*Q*b));
%%%%%%%%%%   bKERNb=b'*KERN*b;
%%%%%%%%%%   if sig_psi>10
%%%%%%%%%%     [psi,acceptpsi]=psi_car2_uni(K,psi,inv_sqr_tau,bKERNb,eigen);
%%%%%%%%%%   else
%%%%%%%%%%     [psi,acceptpsi]=psi_car2(K,psi,inv_sqr_tau,bKERNb,eigen,sig_psi);
%%%%%%%%%%   end;

  bQb=0;
  bKERNb=0;
  if psi>0
    for l=1:K
      b_nah_sum=0;
      for k=1:n_plz(l)
        b_nah_sum=b_nah_sum+b(n_mat(l,k));
      end;
      bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)-b_nah_sum);
    end;
    bQb=psi*bKERNb+(b'*b);
  else
    for l=1:K
      b_nah_sum=0;
      for k=1:n_plz(l)
        b_nah_sum=b_nah_sum+b(n_mat(l,k));
      end;
      bKERNb=bKERNb+b(l)*(b(l)*(n_plz(l)-1)+b_nah_sum);
    end;
    bQb=-psi*bKERNb+(b'*b);
  end;
  if bQb>0
    inv_sqr_tau=gamrnd((K/2)-1,2/bQb);
  end;
  
  if sig_psi>10
    [psi,acceptpsi]=psi_car3_uni_both(K,psi,inv_sqr_tau,bKERNb,eigen_pos,eigen_neg,n_plz,n_mat,b);
  else
    [psi,acceptpsi]=psi_car3_both(K,psi,inv_sqr_tau,bKERNb,eigen_pos,eigen_neg,sig_psi,n_plz,n_mat,b);
  end;

  if acceptpsi==1
%%%%%%%%%%     acceptpsi_cum=acceptpsi_cum+1;
%%%%%%%%%%     Q=eye(K)+psi*KERN;
    if psi>0
      Q=eye(K)-psi*KERN_pos;
    else
      Q=eye(K)-psi*KERN_neg;
    end;
  end;
  sqr_sigma=sqr_sigma_probit(sqr_sigma,eta_Z,C,trips_per_cluster,M);
  sigma=sqrt(sqr_sigma(X(:,2)));
  invSigma=diag(1./sqr_sigma(X(:,2)));
  BinvSigma=zeros(n,K);
  for i=1:n
    BinvSigma(i,X(i,1))=invSigma(i,i);
  end;
  BinvSigma= BinvSigma';
  BinvSigmaB=zeros(K);
  for i=1:n
    BinvSigmaB(X(i,1),X(i,1))=BinvSigmaB(X(i,1),X(i,1))+1/sqr_sigma(X(i,2));
  end;


  if mod(r,step)==0
    anzahl=anzahl+1
    div=r/step;
    out_b(div,:)=b';
    out_a(div,:)=alpha';
    out_sqr_sigma(div,:)=sqr_sigma;
    out_sqr_tau(div)=1/inv_sqr_tau;
    out_phi(div)=psi/(1-abs(psi));
    out_psi(div)=psi;
  end;
  acceptpsi_cum=acceptpsi_cum+acceptpsi;
end;
disp('sig_psi  ');sig_psi
acceptpsi_cum=acceptpsi_cum/R;
disp('acceptpsi_cum   ');acceptpsi_cum
if sig_psi>10
  disp('sig_psi>10 => uni(0,1) for proposal psi is used');
else
  disp('sig_psi<10 => norm(psi_previous,sig_psi) for proposal psi is used');
end;

a_mean=sum(out_a)/R_rec;
b_mean=sum(out_b)/R_rec;
sqr_sigma_mean=sum(out_sqr_sigma)/R_rec;
sqr_tau_mean=sum(out_sqr_tau)/R_rec;
psi_mean=sum(out_psi)/R_rec;
phi_mean1=sum(out_phi)/R_rec;
phi_mean2=psi_mean/(1-abs(psi_mean));
a_mode=zeros(p,1);
b_mode=zeros(K,1);
sqr_sigma_mode=zeros(M,1);
sqr_tau_mode=0;
psi_mode=0;
phi_mode=0;
a_quantile=zeros(p,5);
b_quantile=zeros(K,5);
sqr_sigma_quantile=zeros(M,5);
sqr_tau_quantile=zeros(1,5);
psi_quantile=zeros(1,5);
phi_quantile=zeros(1,5);

figure(1)
for i=1:p
  subplot(floor(sqrt(p))+(p>floor(sqrt(p))*(floor(sqrt(p))+1)),floor(sqrt(p))+(p>(floor(sqrt(p))^2)),i);
%%%%%%%%%%   figure(i)
  plot(1:R_rec,out_a(:,i),'b');
  hold on 
  plot(1:R_rec,alpha_true(i)+zeros(1,R_rec),'g--')
%%%%%%%%%%   grid on
  title(['\fontsize{6}',alpha_lab(i,:)]);
  if i~=p
    set(gca,'XTickLabel',[]);
  end
end;

figure(2)
for i=1:p
  subplot(floor(sqrt(p))+(p>floor(sqrt(p))*(floor(sqrt(p))+1)),floor(sqrt(p))+(p>(floor(sqrt(p))^2)),i);
%%%%%%%%%%   figure(p+i)
  x=sort(out_a(:,i));
  [h,f,tt]=density(x);
  a_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  a_mode(i)=tt(ind);
  plot(tt,f,'b');
  hold on
  axis([tt(1) tt(max(size(tt))) 0 1.05*maxim])
%%%%%%%%%%   plot(alpha_true(i)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(a_mode(i)+zeros(1,100),linspace(0,maxim,100),'b');
%%%%%%%%%%   grid on
  plot(a_quantile(i,2)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(a_quantile(i,4)+zeros(1,100),linspace(0,maxim,100),'g--');
  set(gca,'FontSize',6);
  title(['\fontsize{6}',alpha_lab(i,:)]); 
end;
%%%%%%%%%% adjust = .85;
%%%%%%%%%% h = findobj(gcf,'type','axes');
%%%%%%%%%% for i = 1:length(h)
%%%%%%%%%%   pos = get(h(i),'position');
%%%%%%%%%%   pos(4) = pos(4)*adjust;
%%%%%%%%%%   set(h(i),'position',pos);
%%%%%%%%%% end;

figure(3);
for i=1:K
  subplot(floor(sqrt(K))+(K>floor(sqrt(K))*(floor(sqrt(K))+1)),floor(sqrt(K))+(K>(floor(sqrt(K))^2)),i);
  plot(1:R_rec,out_b(:,i),'b');
  hold on 
  plot(1:R_rec,b_true(i)+zeros(1,R_rec),'g--')
%%%%%%%%%%   hold on
%%%%%%%%%%   grid on
  if i~=K
    set(gca,'XTickLabel',[]);
  end
end;

figure(4);
for i=1:K
  subplot(floor(sqrt(K))+(K>floor(sqrt(K))*(floor(sqrt(K))+1)),floor(sqrt(K))+(K>(floor(sqrt(K))^2)),i);
  x=sort(out_b(:,i));
  [h,f,tt]=density(x);
  b_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  b_mode(i)=tt(ind);
  plot(tt,f,'b');
  axis([tt(1) tt(max(size(tt))) 0 1.05*maxim])
%%%%%%%%%%   plot(b_true(i)+zeros(1,100),linspace(0,maxim,100),'g--');
  hold on
  plot(b_mode(i)+zeros(1,100),linspace(0,maxim,100),'b');
  plot(b_quantile(i,2)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(b_quantile(i,4)+zeros(1,100),linspace(0,maxim,100),'g--');
%%%%%%%%%%   grid on
  set(gca,'FontSize',6);
end;
%%%%%%%%%% h = findobj(gcf,'type','axes');
%%%%%%%%%% for i = 1:length(h)
%%%%%%%%%%   pos = get(h(i),'position');
%%%%%%%%%%   pos(4) = pos(4)*adjust;
%%%%%%%%%%   set(h(i),'position',pos);
%%%%%%%%%% end;

figure(5)
subplot(2,1,1);
plot(1:R_rec,out_sqr_tau,'b');
hold on 
%%%%%%%%%% plot(1:R_rec,sqr_tau_true+zeros(1,R_rec),'g--')
grid on
title('simulations of \tau^2');
subplot(2,1,2);
plot(1:R_rec,out_psi,'b');
hold on 
plot(1:R_rec,psi_true+zeros(1,R_rec),'g--')
grid on
title('simulations of \psi = \phi / (1+|\phi|)');

figure(6)
subplot(1,2,1);
x=sort(out_sqr_tau);
[h,f,tt]=density(x);
sqr_tau_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
sqr_tau_mode=tt(ind);
plot(tt,f,'b');
hold on
%%%%%%%%%% plot(sqr_tau_true+zeros(1,100),linspace(0,maxim,100),'g--');
plot(sqr_tau_mode+zeros(1,100),linspace(0,maxim,100),'b');
plot(sqr_tau_quantile(2)+zeros(1,100),linspace(0,maxim,100),'g--');
plot(sqr_tau_quantile(4)+zeros(1,100),linspace(0,maxim,100),'g--');
grid on
title('posterior density of \tau^2');
subplot(1,2,2);
x=sort(out_psi);
[h,f,tt]=density(x);
psi_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
psi_mode=tt(ind);
plot(tt,f,'b');
hold on
axis([-1.2 1.2 0 1.05*maxim])
%%%%%%%%%% %%%%%%%%%% plot(psi_true+zeros(1,100),linspace(0,maxim,100),'g--');
plot(psi_mode+zeros(1,100),linspace(0,maxim,100),'b');
plot(psi_quantile(2)+zeros(1,100),linspace(0,maxim,100),'g--');
plot(psi_quantile(4)+zeros(1,100),linspace(0,maxim,100),'g--');
grid on
title('posterior density of \psi = \phi / (1+|\phi|)');

figure(7)
subplot(2,1,1);
plot(1:R_rec,out_phi,'b');
hold on 
plot(1:R_rec,phi_true+zeros(1,R_rec),'g--')
hold on
grid on
title('simulations of \phi (transformed from simulations of \psi)'); 
subplot(2,1,2);
x=sort(out_phi);
[h,f,tt]=density(x);
phi_quantile=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
[maxim,ind]=max(f);
phi_mode=tt(ind);
plot(tt,f,'b');
hold on
plot(phi_true+zeros(1,100),linspace(0,maxim,100),'g--');
plot(phi_mode+zeros(1,100),linspace(0,maxim,100),'b');
%plot(phi_mode2+zeros(1,100),linspace(0,maxim,100),'r.');
grid on
title('posterior density of \phi');

figure(8)
for i=2:M
  subplot(floor(sqrt(M-1))+(M-1>floor(sqrt(M-1))*(floor(sqrt(M-1))+1)),floor(sqrt(M-1))+(M-1>(floor(sqrt(M-1))^2)),i-1);
  plot(1:R_rec,out_sqr_sigma(:,i),'b');
  hold on 
%%%%%%%%%%   plot(1:R_rec,sqr_sigma_true(i)+zeros(1,R_rec),'g--','LineWidth',3)
  grid on
%%%%%%%%%%   hold off
  title(['\sigma^2_{',num2str(i),'}']);
  if i~=M
    set(gca,'XTickLabel',[]);
  end
end;

figure(9)
for i=2:M
  subplot(floor(sqrt(M-1))+(M-1>floor(sqrt(M-1))*(floor(sqrt(M-1))+1)),floor(sqrt(M-1))+(M-1>(floor(sqrt(M-1))^2)),i-1);
  x=sort(out_sqr_sigma(:,i));
  [h,f,tt]=density(x);
  sqr_sigma_quantile(i,:)=[x(round(R_rec*0.025)),x(round(R_rec*0.05)),x(round(R_rec*0.5)),x(round(R_rec*0.95)),x(round(R_rec*0.975))];
  [maxim,ind]=max(f);
  sqr_sigma_mode(i)=tt(ind);
  plot(tt,f,'b');
  hold on
  axis([tt(1) tt(max(size(tt))) 0 1.05*maxim])
%%%%%%%%%%   plot(sqr_sigma_true(i)+zeros(1,100),linspace(0,maxim,100),'g--','LineWidth',3);
%%%%%%%%%%   hold on
  plot(sqr_sigma_mode(i)+zeros(1,100),linspace(0,maxim,100),'b');
  plot(sqr_sigma_quantile(i,2)+zeros(1,100),linspace(0,maxim,100),'g--');
  plot(sqr_sigma_quantile(i,4)+zeros(1,100),linspace(0,maxim,100),'g--');
%%%%%%%%%%   hold on
%%%%%%%%%%   grid on
  title(['\sigma^2_{',num2str(i),'}']);
end;

%%%%%%%%%% 
%%%%%%%%%% HIER BLOCK MIT COMPARIZON MAPS FR b UND c FR REGULRE GITTER
%%%%%%%%%% 

figure(10)
for i=1:p
  subplot(floor(sqrt(p))+(p>floor(sqrt(p))*(floor(sqrt(p))+1)),floor(sqrt(p))+(p>(floor(sqrt(p))^2)),i);
  acf_plot(out_a(:,i),floor(R_rec/10),['\fontsize{6}',alpha_lab(i,:)]);
  set(gca,'FontSize',6);
  grid off;
  if i~=p
    set(gca,'XTickLabel',[]);
  end;
end;
%%%%%%%%%% h = findobj(gcf,'type','axes');
%%%%%%%%%% for i = 1:length(h)
%%%%%%%%%%   pos = get(h(i),'position');
%%%%%%%%%%   pos(4) = pos(4)*adjust;
%%%%%%%%%%   set(h(i),'position',pos);
%%%%%%%%%% end;

figure(11)
subplot(1,2,1)
acf_plot(out_sqr_tau,floor(R_rec/10),'\tau^2');
%%%%%%%%%% hold off
subplot(1,2,2)
acf_plot(out_psi,floor(R_rec/10),'\psi');


figure(12)
for i=2:M
  subplot(1,M-1,i-1)
  acf_plot(out_sqr_sigma(:,i),floor(R_rec/10),['\sigma^2_{',num2str(i),'}']);
  hold off;
end;


out.data=data;
out.out_a=out_a;
out.out_b=out_b;
out.out_sqr_sigma=out_sqr_sigma;
out.out_sqr_tau=out_sqr_tau;
out.out_psi=out_psi;
% out.out_phi=out.out_psi./(1+abs(out.out_psi));
out.a_mean=a_mean';
out.b_mean=b_mean';
out.sqr_sigma_mean=sqr_sigma_mean';
out.sqr_tau_mean=sqr_tau_mean;
out.psi_mean=psi_mean;
out.phi_mean1=phi_mean1;
out.phi_mean2=phi_mean2;
out.a_mode=a_mode;
out.b_mode=b_mode;
out.sqr_sigma_mode=sqr_sigma_mode;
out.sqr_tau_mode=sqr_tau_mode;
out.psi_mode=psi_mode;
out.phi_mode=phi_mode;
out.a_quantile=a_quantile;
out.b_quantile=b_quantile;
out.sqr_sigma_quantile=sqr_sigma_quantile;
out.sqr_tau_quantile=sqr_tau_quantile;
out.psi_quantile=psi_quantile;
out.phi_quantile=phi_quantile;
out.sig_psi=sig_psi;
out.acceptpsi_cum=acceptpsi_cum;
out.C=C;
out.trips_per_cluster=trips_per_cluster;
out.R=R;
out.step=step;
out.K_MH=K_MH;
toc;
t=toc
out.t=t;


% end of function sim_probit_neu_latent.m
